==> distfun-0.3-1-src.zip: distfun/src/cdflib/bpser.c <==
/* bpser.f -- translated by f2c (version 20100827).
You must link the resulting object file with libf2c:
on Microsoft Windows system, link with libf2c.lib;
on Linux or Unix systems, link with .../path/to/libf2c.a -lm
or, if you install libf2c.a in a standard place, with -lf2c -lm
-- in that order, at the end of the command line, as in
cc *.o -lf2c -lm
Source for libf2c is in /netlib/f2c/libf2c.zip, e.g.,

http://www.netlib.org/f2c/libf2c.zip
*/

#include "cdflib.h"

double bpser_(double *a, double *b, double *x, double *
	eps)
{
	/* System generated locals */
	int i__1;
	double ret_val;

	/* Builtin functions */
	double log(double), exp(double), pow_dd(double *, double *
		);

	/* Local variables */
	static double c__;
	static int i__, m;
	static double n, t, u, w, z__, a0, b0, apb, tol, sum;

	/* ----------------------------------------------------------------------- */
	/*     POWER SERIES EXPANSION FOR EVALUATING IX(A,B) WHEN B .LE. 1 */
	/*     OR B*X .LE. 0.7.  EPS IS THE TOLERANCE USED. */
	/* ----------------------------------------------------------------------- */
	/*     .. Scalar Arguments .. */
	/*     .. */
	/*     .. Local Scalars .. */
	/*     .. */

	/*     .. */
	/*     .. Intrinsic Functions .. */
	/*     .. */
	/*     .. Executable Statements .. */

	ret_val = 0.;
	if (*x == 0.) {
		return ret_val;
	}
	/* ----------------------------------------------------------------------- */
	/*            COMPUTE THE FACTOR X**A/(A*BETA(A,B)) */
	/* ----------------------------------------------------------------------- */
	a0 = min(*a,*b);
	if (a0 < 1.) {
		goto L10;
	}
	z__ = *a * log(*x) - betaln_(a, b);
	ret_val = exp(z__) / *a;
	goto L100;
L10:
	b0 = max(*a,*b);
	if (b0 >= 8.) {
		goto L90;
	}
	if (b0 > 1.) {
		goto L40;
	}

	/*            PROCEDURE FOR A0 .LT. 1 AND B0 .LE. 1 */

	ret_val = pow_dd(x, a);
	if (ret_val == 0.) {
		return ret_val;
	}

	apb = *a + *b;
	if (apb > 1.) {
		goto L20;
	}
	z__ = gam1_(&apb) + 1.;
	goto L30;
L20:
	u = *a + *b - 1.;
	z__ = (gam1_(&u) + 1.) / apb;

L30:
	c__ = (gam1_(a) + 1.) * (gam1_(b) + 1.) / z__;
	ret_val = ret_val * c__ * (*b / apb);
	goto L100;

	/*         PROCEDURE FOR A0 .LT. 1 AND 1 .LT. B0 .LT. 8 */

L40:
	u = gamln1_(&a0);
	m = (int) (b0 - 1.);
	if (m < 1) {
		goto L60;
	}
	c__ = 1.;
	i__1 = m;
	for (i__ = 1; i__ <= i__1; ++i__) {
		b0 += -1.;
		c__ *= b0 / (a0 + b0);
		/* L50: */
	}
	u = log(c__) + u;

L60:
	z__ = *a * log(*x) - u;
	b0 += -1.;
	apb = a0 + b0;
	if (apb > 1.) {
		goto L70;
	}
	t = gam1_(&apb) + 1.;
	goto L80;
L70:
	u = a0 + b0 - 1.;
	t = (gam1_(&u) + 1.) / apb;
L80:
	ret_val = exp(z__) * (a0 / *a) * (gam1_(&b0) + 1.) / t;
	goto L100;

	/*            PROCEDURE FOR A0 .LT. 1 AND B0 .GE. 8 */

L90:
	u = gamln1_(&a0) + algdiv_(&a0, &b0);
	z__ = *a * log(*x) - u;
	ret_val = a0 / *a * exp(z__);
L100:
	if (ret_val == 0. || *a <= *eps * .1) {
		return ret_val;
	}
	/* ----------------------------------------------------------------------- */
	/*                     COMPUTE THE SERIES */
	/* ----------------------------------------------------------------------- */
	sum = 0.;
	n = 0.;
	c__ = 1.;
	tol = *eps / *a;
L110:
	n += 1.;
	c__ = c__ * (.5 - *b / n + .5) * *x;
	w = c__ / (*a + n);
	sum += w;
	if (fabs(w) > tol) {
		goto L110;
	}
	ret_val *= *a * sum + 1.;
	return ret_val;
} /* bpser_ */

==> distfun-0.5-1-src.zip: distfun/src/cdflib/bpser.c <==
/* bpser.f -- translated by f2c (version 20100827).
with manual tuning by Michael Baudin (2012)
*/

#include <math.h>

#include "cdflib.h"
#include "cdflib_private.h" 

double cdflib_bpser(double a, double b, double x, double eps)
{
	/* System generated locals */
	int i__1;
	double ret_val;

	/* Local variables */
	static double c__;
	static int i__, m;
	static double n, t, u, w, z__, a0, b0, apb, tol, sum;

	/* ----------------------------------------------------------------------- */
	/*     POWER SERIES EXPANSION FOR EVALUATING IX(A,B) WHEN B .LE. 1 */
	/*     OR B*X .LE. 0.7.  EPS IS THE TOLERANCE USED. */
	/* ----------------------------------------------------------------------- */

	ret_val = 0.;
	if (x == 0.) {
		return ret_val;
	}
	/* ----------------------------------------------------------------------- */
	/*            COMPUTE THE FACTOR X**A/(A*BETA(A,B)) */
	/* ----------------------------------------------------------------------- */
	a0 = min(a,b);
	if (a0 < 1.) {
		goto L10;
	}
	z__ = a * log(x) - cdflib_betaln(a, b);
	ret_val = exp(z__) / a;
	goto L100;
L10:
	b0 = max(a,b);
	if (b0 >= 8.) {
		goto L90;
	}
	if (b0 > 1.) {
		goto L40;
	}

	/*            PROCEDURE FOR A0 .LT. 1 AND B0 .LE. 1 */

	ret_val = cdflib_powdd(x, a);
	if (ret_val == 0.) {
		return ret_val;
	}

	apb = a + b;
	if (apb > 1.) {
		goto L20;
	}
	z__ = cdflib_gam1(apb) + 1.;
	goto L30;
L20:
	u = a + b - 1.;
	z__ = (cdflib_gam1(u) + 1.) / apb;

L30:
	c__ = (cdflib_gam1(a) + 1.) * (cdflib_gam1(b) + 1.) / z__;
	ret_val = ret_val * c__ * (b / apb);
	goto L100;

	/*         PROCEDURE FOR A0 .LT. 1 AND 1 .LT. B0 .LT. 8 */

L40:
	u = cdflib_gamln1(a0);
	m = (int) (b0 - 1.);
	if (m < 1) {
		goto L60;
	}
	c__ = 1.;
	i__1 = m;
	for (i__ = 1; i__ <= i__1; ++i__) {
		b0 += -1.;
		c__ *= b0 / (a0 + b0);
		/* L50: */
	}
	u = log(c__) + u;

L60:
	z__ = a * log(x) - u;
	b0 += -1.;
	apb = a0 + b0;
	if (apb > 1.) {
		goto L70;
	}
	t = cdflib_gam1(apb) + 1.;
	goto L80;
L70:
	u = a0 + b0 - 1.;
	t = (cdflib_gam1(u) + 1.) / apb;
L80:
	ret_val = exp(z__) * (a0 / a) * (cdflib_gam1(b0) + 1.) / t;
	goto L100;

	/*            PROCEDURE FOR A0 .LT. 1 AND B0 .GE. 8 */

L90:
	u = cdflib_gamln1(a0) + cdflib_algdiv(a0, b0);
	z__ = a * log(x) - u;
	ret_val = a0 / a * exp(z__);
L100:
	if (ret_val == 0. || a <= eps * .1) {
		return ret_val;
	}
	/* ----------------------------------------------------------------------- */
	/*                     COMPUTE THE SERIES */
	/* ----------------------------------------------------------------------- */
	sum = 0.;
	n = 0.;
	c__ = 1.;
	tol = eps / a;
L110:
	n += 1.;
	c__ = c__ * (.5 - b / n + .5) * x;
	w = c__ / (a + n);
	sum += w;
	if (fabs(w) > tol) {
		goto L110;
	}
	ret_val *= a * sum + 1.;
	return ret_val;
}
